@@ -23,6 +23,7 @@ has orphans        => sub { {} };
 has process_table  => sub { {} };
 has collected_info => sub { [] };
 has 'handler';
+has emit_from_sigchld => 1;
 
 my $singleton;
 
@@ -55,6 +56,7 @@ sub enable {
         while ((my $pid = waitpid(-1, WNOHANG)) > 0) {
           $singleton->add_collected_info($pid, $?, $!);
         }
+        $singleton->consume_collected_info() if ($singleton->emit_from_sigchld());
       }
     });
 }
@@ -133,7 +135,7 @@ sub contains {
   $singleton->all->grep(sub { $_->pid eq $pid })->size == 1;
 }
 
-sub reset { @{+shift}{qw(events orphans process_table collected_info handler)} = ({}, {}, {}, [], undef) }
+sub reset { @{+shift}{qw(events orphans process_table collected_info handler emit_from_sigchld)} = ({}, {}, {}, [], undef, 1) }
 
 # XXX: This should be replaced by PR_GET_CHILD_SUBREAPER
 sub disable_subreaper {
